/**
 * Copyright (c) 2008 - 2012 10gen, Inc. <http://10gen.com>
 * <p/>
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * <p/>
 * http://www.apache.org/licenses/LICENSE-2.0
 * <p/>
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.mongodb.tools;

import com.mongodb.DBObject;
import com.mongodb.util.JSON;
import com.mongodb.util.TestCase;
import org.testng.annotations.Test;

import javax.management.JMException;
import java.io.IOException;

public class ConnectionPoolStatTest extends TestCase {

    @Test
    public void testBasic() throws IOException, JMException {
        if (!isAtLeastJava6()) {
            return;
       }
        ConnectionPoolStat stat = new ConnectionPoolStat();
        String stats = stat.getStats();
        assertNotNull(stats);
        DBObject obj = (DBObject) JSON.parse(stats);
        assertNotNull(obj);
    }

    private boolean isAtLeastJava6() {
        String javaSpecificationVersion = System.getProperty("java.specification.version");
        String minorVersionString = javaSpecificationVersion.substring(javaSpecificationVersion.lastIndexOf(".") + 1);
        try {
            return Integer.parseInt(minorVersionString) >= 6;
        } catch (NumberFormatException e) {
            return false;
        }
    }
}
